/*
 * Copyright (C) 2017-2021 Dominique Dhoutaut, Thierry Arrabal, Eugen Dedu.
 *
 * This file is part of BitSimulator.
 *
 * BitSimulator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BitSimulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BitSimulator.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef AGENTS_PURE_FLOODING_PERFECT_RING_ROUTING_AGENT_H_
#define AGENTS_PURE_FLOODING_PERFECT_RING_ROUTING_AGENT_H_

#include <random>
#include "../routing-agent.h"
 

//==============================================================================
//
//          PureFloodingPerfectRingRoutingAgent  (class)
//
//==============================================================================


typedef struct {
  bool control1;
  bool control2;
} Control12_t;

typedef struct {
  int hopCount;
  bool isOnTransmitterRing;
} RingInfo_t;


class PureFloodingPerfectRingRoutingAgent : public RoutingAgent {
 protected:
  map<int,int> alreadySeenPackets;
  map<int, Control12_t> neighb;
  map<int, RingInfo_t> sourceRing;
  static set<int> reachability;

  bool alreadySent1and2;
  distance_t rangeBig, rangeSmall;

 public:
  PureFloodingPerfectRingRoutingAgent(Node *_hostNode);
  virtual ~PureFloodingPerfectRingRoutingAgent();

  virtual void receivePacketFromNetwork(PacketPtr _packet);
  virtual void receivePacketFromApplication(PacketPtr _packet);
  void processPacketAndFwIfOnRing( PacketPtr _packet );
  static void initializeAgent();
  virtual void processPacketSendEvent( PacketPtr _packet );
  
  using PureFloodingPerfectRingSendEvent = CallMethodArgEvent<PureFloodingPerfectRingRoutingAgent, PacketPtr,
    &PureFloodingPerfectRingRoutingAgent::processPacketSendEvent,
    EventType::RING_SEND>;
};


#endif /* AGENTS_PURE_FLOODING_PERFECT_RING_ROUTING_AGENT_H_ */
